/*
 * File:        label-loader.js
 * CVS:         $Id$
 * Description: Load up Label
 * Author:			Allan Jardine
 * Created:		  21-1-08
 * Modified:		$Date$ by $Author$
 * Language:		JavaScript
 * Project:		  Design
 * License:     GNU GPL v2.0
 * 
 * Copyright 2007-2008 Allan Jardine. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2.0 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */


/*
 * Object:  Label
 * Purpose:	Label constructor
 * Returns:	-
 * Inputs:	-
 * Notes:   -
 */
if ( typeof LabelLoader != 'undefined' )
{
	LabelLoader = undefined;
}

var LabelLoader = {};
LabelLoader.bLoadingComplete = false;


/*
 * Function: LabelLoader.fnLoadFile
 * Purpose:  Load a new file into the DOM
 * Returns:  -
 * Inputs:   
 *  string:sFile - file to load 
 *  string:sType - type of file 'css' or 'js'
 * Notes:    -
 */
LabelLoader.fnLoadFile = function ( sFile, sType )
{
	var nElement;
	
	if ( sType == 'css' )
	{
		nElement = document.createElement('link');
		nElement.type = 'text/css';
		nElement.rel = 'stylesheet';
		nElement.href = sFile;
		nElement.media = 'screen';
		document.getElementsByTagName('head')[0].appendChild( nElement );
	}
	else if ( sType == 'image' )
	{
		var imImage = new Image( 1, 1 );
		imImage.src = sFile;
	}
	else
	{
		nElement = document.createElement( 'script' );
		nElement.setAttribute( 'language', 'JavaScript' );
		nElement.setAttribute( 'src', sFile );
		document.getElementsByTagName('body')[0].appendChild( nElement );
	}
}


/*
 * Function: LabelLoader.fnLoad
 * Purpose:  Start the loading
 * Returns:  -
 * Inputs:   -
 * Notes:    -
 */
LabelLoader.fnLoad = function ()
{
	/* Check to see if already loaded */
	if ( typeof Label == 'object' &&
	     typeof jsCore == 'object' )
	{
		Label.fnCreateLabel();
		return 0;
	}
	
	var nLoading = document.createElement( 'div' );
	nLoading.style.position = 'absolute';
	nLoading.style.top = '0';
	nLoading.style.left = '0';
	nLoading.style.color = 'white';
	nLoading.style.padding = '5px 10px';
	nLoading.style.fontSize = '11px';
	nLoading.style.fontFamily = '"Lucida Grande", Verdana, Arial, Helvetica, sans-serif';
	nLoading.style.zIndex = '32501';
	nLoading.style.backgroundColor = '#a2392d';
	nLoading.setAttribute( 'id', 'LoadingLabel' );
	nLoading.appendChild( document.createTextNode( 'Loading Label...' ) );
	
	document.getElementsByTagName('body')[0].insertBefore( 
		nLoading, document.body.childNodes[0] );
	
	/* Images - set up for caching (after the main scrips have loaded) */
	var aColours = [ 'yellow', 'blue', 'green', 'red', 'white', 'black' ];
	var aImages = [ 'background_strip.png', 'bottom.png', 
		'bottom_left_corner.png', 'bottom_right_corner.png', 'left.png', 
		'right.png', 'top_left_corner.png', 'top_right_corner.png' ];
	
	/* Deploy */
	LabelLoader.fnLoadFile( 'http://www.sprymedia.co.uk/design/label/media/css/label.css', 'css' );
	LabelLoader.fnLoadFile( 'http://www.sprymedia.co.uk/design/label/media/js/label-complete.js', 'js' );
	
	for ( var i=0 ; i<aColours.length ; i++ )
	{
		for ( var j=0 ; j<aImages.length ; j++ )
		{
			LabelLoader.fnLoadFile( 
				'http://www.sprymedia.co.uk/design/label/media/images/'+aColours[i]+'/'+aImages[j],
	      'image' );
		}
	}
	
	/* Localhost testing */
	//LabelLoader.fnLoadFile( '/design/label/media/css/label.css', 'css' );
	//
	//LabelLoader.fnLoadFile( '/design/label/media/js/js-core.js', 'js' );
	//LabelLoader.fnLoadFile( '/design/label/media/js/design-core.js', 'js' );
	//LabelLoader.fnLoadFile( '/design/label/media/js/remoting.js', 'js' );
	//LabelLoader.fnLoadFile( '/design/label/media/js/label.js', 'js' );
	//
	//for ( var i=0 ; i<aColours.length ; i++ )
	//{
	//	for ( var j=0 ; j<aImages.length ; j++ )
	//	{
	//		LabelLoader.fnLoadFile( 
	//			'/design/label/media/images/'+aColours[i]+'/'+aImages[j], 'image' );
	//	}
	//}
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Initialisation - poll until all the required objects are available and 
 *   then initalise the crosshair.
 */
LabelLoader.fnLoad();

